/**
 * <p>Title: OWAMP Protocol</p>
 * <p>Description: This project contains the classes comum to Sender and Receiver.</p>
 * <p>Copyright: Copyright (c) 2005</p>
 * <p>Company: University of Aveiro / Institute of Telecommunications - Aveiro pole</p>
 * @authors: Helder Veiga and Teresa Pinho
 * @version: 2.1
 * @Last updated: 9-12-2005
 *
 * Copyright (C) 2005  Helder Veiga and Teresa Pinho
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.  A HTML version of
 * the GNU General Public License can be seen at
 * http://www.gnu.org/licenses/gpl.html.
 *
 */


import java.net.*;
import java.util.Random;


public class OWAMP_SessionTerminal extends OWAMP_Base{

  // Constructors

  public OWAMP_SessionTerminal(int[] portRange) {

    rand = new Random();
    this.portRange = portRange;

  }


  // Instance variables

  protected Socket socketToCS = new Socket();                   // Represents the socket connection to the Client or Server.
  protected byte[] sid = new byte[16];
  protected int[] portRange = null;                             // Represents the UDP port range to be used during test sessions.
  protected RequestSessionReceiver dataRS;                      // Represents the RequestSession datagram.
  protected ScheduleSlotDescriptionsReceiver[] arrayOfdataSSD;  // Represents the arrayOfdataSSD of the RequestSession datagram
                                                                // that is going to be processed.
  protected DatagramSocket ds;                                  // The DatagramSocket for UDP Socket.
  protected String element;                                     // Represents the element in use (Sender or Receiver).


  // Instance methods

  /**
   * Method to receive the packet Request Session send by a client.
   * To each Request-Session message, an OWAMP Session-Sender/Receiver MUST respond with an Accept-Session message.
   * @param accept short - specify if the Session-Sender/Receiver accepts the request. In this message, zero in the
   * Accept field means that the Session-Sender/Receiver is willing to conduct the session.  A value of 1 indicates
   * rejection of the request.  All other values are reserved.
   * @return boolean - This method returns true if the request operation was ok, otherwise it returns false.
   */

  public boolean requestSessionResponse(byte accept){
    return true;
  }


  /**
   * Having requested one or more test sessions and received affirmative Accept-Session responses, an OWAMP
   * client may start the execution of the requested test sessions by sending a Start-Sessions message to
   * the server.
   * @param accept byte - If Accept is 1, the Start-Sessions request was rejected. Zero means that the
   * command was accepted.  All other values are reserved.  The server MAY and the client SHOULD close the
   * connection in the case of a negative response.
   * @return boolean -  It returns true if everything goes ok, otherwise it returns false.
   */

  public boolean startingTestSessionsResponse(byte accept){ // process Starting Test Sessions Response
    return true;
  }


  /**
   * This method is used to determine which packet was received and process it.
   * The first byte read represents the type of the packet.
   * @return boolean - It returns true if everything goes ok, otherwise it returns false.
   */

  public boolean processPacketReceived(){

    int readn = -1;  // number of bytes read

    switch((int)(buffer[0] & 0xFF)){
      case 1:  // Request-Session
        readn = receivePacket(socketToCS, buffer, 1, RequestSession.PACKET_LENGTH -1);   // receivePacket (Socket socket, byte[] _buffer, int off, int len)
        if(readn == (RequestSession.PACKET_LENGTH -1)){   // Packet Request Session
          System.out.println("Request Session from "+element+" on "+socketToCS);
          System.out.println();

          byte accept = 0;

          if (!requestSessionResponse(accept)) { // requestSession(byte accept)
            System.err.println("Error: Error in Request-Session operation.");
            return false;
          }

          return true;

        }else
          System.err.println("Error: The packet received in socket "+socketToCS+" is not a Request Session packet.\n");
        break;
      case 2:  // Starting Test Sessions
        readn = receivePacket(socketToCS,buffer,1,StartingTestSessions.PACKET_LENGTH -1);   // receivePacket (Socket socket, byte[] _buffer, int off, int len)
        if(readn == (StartingTestSessions.PACKET_LENGTH -1)){   // Packet Starting Test Sessions
          System.out.println("Starting Test Sessions from "+element+" on "+socketToCS);
          System.out.println();

          byte accept = 0;
          if (!startingTestSessionsResponse(accept)) {  // startingTestSessionsResponse(byte accept)
            System.err.println("Error: Error in Starting Test Sessions operation.");
            return false;
          }
          return true;
        }else
          System.err.println("Error: The packet received in socket "+socketToCS+" is not a Starting Test Sessions packet.\n");

        break;
      case 3:  // Stop Sessions. If a SS is received before the Starting Test session command, this session should be finished.
        readn = receivePacket(socketToCS,buffer,1,StopSessions.PACKET_LENGTH -1);   // receivePacket (Socket socket, byte[] _buffer, int off, int len)
        if(readn == (StopSessions.PACKET_LENGTH -1)){   // Packet Stop Sessions

          System.out.println("Stop Sessions from "+element+" on "+socketToCS);
          System.out.println();
          System.err.println("Error: This Session will not be started.");
          closeSocket(socketToCS);
          if(!ds.isClosed())
            ds.close();
          return true;
        }else
          System.err.println("Error: The packet received in socket "+socketToCS+" is not a Stop Sessions packet.\n");

        break;
      case 4:   // Fetch-Session
        if(!processFetchSession()){
          System.err.println("Error: Error processing Fetch session data operation.");
          closeSocket(socketToCS);
          return false;
        }else
          return true;
      default:
        System.err.println("Error: The packet received doesn't exits in the protocol.");
    }

    // If we reach this point it means that the packet received doesn't respects the protocol or an error occurred.
    // So, we should stop this OWAMP-Control session.

    if(readn > 0)   // Packet received has a valid type but not a valid length.
      System.err.println("Error: The packet received doesn't exits in the protocol.\n");

    if(readn == 0)  // Timeout to wait occurred.
      System.err.println("Error: Timeout to wait for the packet occurred.\n");

    System.err.println("This session will be finished.");
    System.err.println();
    closeSocket(socketToCS);

    return false;

  }


  /**
   * This method is used, by the Sender/Receiver, to process the Fetch Session.
   */

  public boolean processFetchSession(){
    return true;
  }

}
